function [X,f,nn] = algNlc(A,X,n,k,maxIt,tol,verbose)
% Newton-Grassmann alg with local coordinate representation.
% function [X,f,nn] = algNlc(A,X,n,k,maxIt,tol)
% 
% Copyright 2008.
% Berkant Savas, Link�ping University.

%% Initiate and set the data for the prodGrass objekt. 
G = prodGrass(n,k);
G = set(G,'data',X,'base',X);

%% Local coordinate Newton-Grassmann alg.
it = 0;
done = 0;
%tic
while ~done && it < maxIt
    it=it+1;
    [g,H] = compDeriv(A,G,n,k,'lc');
    DX    = solveNeqn(g,H);
    nn(it,1) = grassNorm(g);
    
    f(it,1) = F(A,G);    
    G = set(G,'rtan',DX);    
    stepVal(it) = fminbnd(@(t) lineFunc(A,G,t,'lc'),0,1);
    if nn(it)/abs(f(it)) < 1e-4
        stepVal(it) = 1;
    end    
    G = move(G,1,'pb'); 
    if nn(it)/abs(f(it)) < tol
      done   = 1;
    end
%    disp([' Iteration: ', int2str(it) '  Rel. norm:  ' num2str(nn(it)/abs(f(it)))])
end
% toc
if nargin==7 & verbose==1
    disp(['Newton, iterations: ', int2str(it) '  Rel. norm:  ' num2str(nn(it)/abs(f(it)))])
    disp(' ')
end

f = -f;
X = get(G,'data');
