function[F,norm_phi,norm_gradient]=norm_phi_gradientLocC(C,r) 

% Compute the Grassmann gradient using the elements of the tensor C.  
% C is a 3-tensor obtained by taking at least one block Krylov step
%   starting from one approximation of dimension r  
%  (Proposition 4.2 and Figure 1 in Eldén-Dehghan A Krylov-Schur-Schur like
%  method...., arXiv (2020))

n=size(C);
F=C(1:r(1),1:r(2),1:r(3)); 
norm_phi=norm(F)^2/2;
 

%% Compute the gradient 

if r(1)==1 & r(2)==1 & r(3)==1          % F is no longer tensor but scalar
    g{1}=-double(C(r(1)+1:n(1),1,1))*F;
    g{2}=-double(C(1,r(2)+1:n(2),1))*F;
    g{3} = -double(C(1:r(1),1:r(2),r(3)+1:end))*F;    % < F^3_perp , F >_3
elseif r(3)==1     % The third mode has disappeared in F and C:
    % Name      Size            Bytes  Class     Attributes
    % F         2x2               400  tensor
    g{1} = -double(ttt(C(r(1)+1:n(1),1:r(2),1:r(3)),F,[2]));    % < F^1_perp , F >_1
    g{2} = -double(ttt(C(1:r(1),r(2)+1:end,1:r(3)),F,[1]));    % < F^2_perp , F >_2
    g{3} = -double(ttt(C(1:r(1),1:r(2),r(3)+1:end),F,[1,2]));    % < F^3_perp , F >_3
else  % r(3)>1 
    g{1} = -double(ttt(C(r(1)+1:n(1),1:r(2),1:r(3)),F,[2 3]));    % < F^1_perp , F >_1
    g{2} = -double(ttt(C(1:r(1),r(2)+1:end,1:r(3)),F,[1,3]));    % < F^2_perp , F >_2
    dimC=size(C);
    if dimC(3)>r(3)
        g{3} = -double(ttt(C(1:r(1),1:r(2),r(3)+1:end),F,[1,2]));    % < F^3_perp , F >_3
    else
        g{3}=0;
    end
end

norm_gradient = 0;
for i = 1:ndims(C)
    norm_gradient=norm_gradient+ norm(g{i},'fro')^2;
end
norm_gradient=sqrt(norm_gradient);


